import re
import os

from pygyat import GYAT2PY_MAPPINGS

"""
Python module for converting pygyat code to python code.
"""


def _ends_in_gyat(word):
    """
    Returns True if word ends in .gyat, else False

    Args:
        word (str):     Filename to check

    Returns:
        boolean: Whether 'word' ends with 'gyat' or not
    """
    return word[-5:] == ".gyat"


def _change_file_name(name, outputname=None):
    """
    Changes *.gyat filenames to *.py filenames. If filename does not end in .gyat,
    it adds .py to the end.

    Args:
        name (str):         Filename to edit
        outputname (str):   Optional. Overrides result of function.

    Returns:
        str: Resulting filename with *.py at the end (unless 'outputname' is
        specified, then that is returned).
    """

    # If outputname is specified, return that
    if outputname is not None:
        return outputname

    # Otherwise, create a new name
    if _ends_in_gyat(name):
        return name[:-5] + ".py"

    else:
        return name + ".py"


def parse_glazes(filename):
    """
    Reads the file, and scans for imports. Returns all the assumed filename
    of all the imported modules (ie, module name appended with ".gyat")

    Args:
        filename (str):     Path to file

    Returns:
        list of str: All imported modules, suffixed with '.gyat'. Ie, the name
        the imported files must have if they are pygyat files.
    """
    infile = open(filename, "r")
    infile_str = ""

    for line in infile:
        infile_str += line

    glazes = re.findall(r"(?<=glaze\s)[\w.]+(?=;|\s|$)", infile_str)
    glazes2 = re.findall(r"(?<=lock in\s)[\w.]+(?=\s+glaze)", infile_str)

    glazes_with_suffixes = [im + ".gyat" for im in glazes + glazes2]

    return glazes_with_suffixes


def parse_file(filepath, filename_prefix, outputname=None, change_imports=None):
    """
    Converts a pygyat file to a python file and writes it to disk.

    Args:
        filename (str):             Path to the pygyat file you want to parse.
        filename_prefix (str):      Prefix to resulting file name (if -c or -k
                                    is not present, then the files are prefixed
                                    with a '.').
        outputname (str):           Optional. Override name of output file. If
                                    omitted it defaults to substituting '.gyat' to
                                    '.py'
        change_imports (dict):      Names of imported pygyat modules, and their
                                    python alternative.
    """
    filename = os.path.basename(filepath)

    infile = open(filepath, "r")
    outfile = open(filename_prefix + _change_file_name(filename, outputname), "w")

    # Read file to string
    infile_str_raw = ""
    for line in infile:
        infile_str_raw += line

    # Fix indentation
    infile_str_indented = ""
    for line in infile_str_raw.split("\n"):
        # Search for comments, and remove for now. Re-add them before writing to
        # result string
        m = re.search(r"[ \t]*(#.*$)", line)

        # Make sure # sign is not inside quotations. Delete match object if it is
        if m is not None:
            m2 = re.search(r"[\"'].*#.*[\"']", m.group(0))
            if m2 is not None:
                m = None

        if m is not None:
            add_comment = m.group(0)
            line = re.sub(r"[ \t]*(#.*$)", "", line)
        else:
            add_comment = ""

        # skip empty lines:
        if line.strip() in ("\n", "\r\n", ""):
            infile_str_indented += add_comment + "\n"
            continue

        # replace anything in mappings.keys() with its value, ignore comments
        # disallow real python
        for key, value in GYAT2PY_MAPPINGS.items():
            deescaped_key = key.replace("\s+", " ")
            line = re.sub(
                r'(?<!["\'#])\b{}\b(?!["\'])'.format(re.escape(value)),
                f"dont_use_{value}_use_{deescaped_key}",
                line,
            )
            line = re.sub(r'(?<!["\'#])\b{}\b(?!["\'])'.format(key), value, line)

        infile_str_indented += line + add_comment + "\n"

    # Change imported names if necessary
    if change_imports is not None:
        for module in change_imports:
            infile_str_indented = re.sub(
                r"(?<=import\\s{})\\b(?!\\s+as\\b)".format(module),
                "{} as {}".format(change_imports[module], module),
                infile_str_indented,
            )
            infile_str_indented = re.sub(
                "(?<=from\\s){}(?=\\s+import)".format(module),
                change_imports[module],
                infile_str_indented,
            )

    outfile.write(infile_str_indented)

    infile.close()
    outfile.close()



import sys
import os

import pygyat.parser
import pygyat.logger


"""
Module for handling imports of PyGyat files in Python code.
"""


def glaze(module_name, globals, logger=None):
    """
    Import (glaze) PyGyat files in Python code. Example:
    
    ``` python
    from pygyat.importing import glaze
    glaze("test_module", globals())

    # Now, 'test_module' is available like any other module:
    test_module.func()
    ```
    
    Args:
        module_name (str):              Name of module to import.
        globals (dict):                 Global namespace (use globals() to get 
                                        this).
        logger (pygyat.logger.Logger)   Optional. Logger object. Mainly used for
                                        debugging.

    Raises:
        ImportError: If no pygyat file for module is found, or pygyat file could
        not be parsed. 
    """
    if logger is None:
        logger = pygyat.logger.Logger()

    logger.log_info("Looking for %s.gyat" % module_name)
    path = _locate_module(module_name, logger)

    logger.log_info("Parsing %s" % path)
    try:
        pygyat.parser.parse_file(path, os.path.join(sys.path[0], "python_"))

        error_during_parsing = None

    except Exception as e:
        error_during_parsing = e

    if error_during_parsing is not None:
        raise ImportError("Error while parsing '%s': %s" % (path, str(e)))

    python_file_path = os.path.join(
        sys.path[0], 
        "python_" + pygyat.parser._change_file_name(module_name, None)
    )

    logger.log_info("Importing %s" % python_file_path)

    # Hacky way of doing global imports of general modules inside a function
    exec("global %s" % module_name, globals)
    exec("import python_%s as %s" % (module_name, module_name), globals)

    # Cleanup
    logger.log_info("Removing %s" % python_file_path)
    os.remove(python_file_path)


def _locate_module(module_name, logger):
    """
    Locate the pygyat file for a given module name.

    Args:
        module_name (str):              Module to look for.
        logger (pygyat.logger.Logger)   Optional. Logger object. Mainly used for
                                        debugging.

    Returns:
        str: Full path of pygyat file associated with module. 
    
    Raises:
        ImportError: If module is not found.
    """
    for path in sys.path:
        logger.log_info("Searching in %s" % path)

        module_path = _traverse_and_find(module_name, path) 

        if module_path is not None:
            logger.log_info("Module found at %s" % module_path)
            break

    if module_path is None:
        raise ImportError("Could not find any pygyat file for %s" % module_name)

    return module_path


def _traverse_and_find(module_name, directory):
    """
    Traverse a directory (recursively), and look for a file named 
    'module_name'.gyat.

    Args:
        module_name (str):              Module to look for.
        directory (str):                Path to directory to traverse.

    Returns:
        str: Full path of pygyat file associated with module, None if no such
        file is found.
    """
    for rootpath, subdirs, files in os.walk(directory):
        for file in files:
            if file == (module_name+".gyat"):
                return os.path.join(rootpath, file)

    return None




import sys


class Logger(object):
    """ Logger class. Writes errors and info to the cmd line. """

    def __init__(self, verbose=False):
        """
        Initialize a logger object.

        Args:
            verbose (boolean):      If True, the logger will print all status 
                                    updates and warnings, in addition to errors
        """
        self.verbose = verbose


    def log_info(self, text):
        """
        Log an info line, prefixed with an [i]. If verbose is False, this does
        nothing.

        Args:
            text (str):             Info text to print.
        """
        if self.verbose: print(" [i]  %s" % text)


    def log_warn(self, text):
        """
        Log a warning, prefixed with an [!]. If verbose is False, this does
        nothing.

        Args:
            text (str):             Warning text to print.
        """
        if self.verbose: print(" [!]  %s" % text)


    def program_header(self):
        """
        Print a header ment to separate info from compiler from the output of 
        the resulting program. If verbose is False, this does nothing.
        """
        if self.verbose: print("\n---- OUPUT FROM PROGRAM ----\n")


    def program_footer(self):
        """
        Print a footer ment to separate the output of the resulting program 
        from the info from compiler. If verbose is False, this does nothing.
        """        
        if self.verbose: print("\n----     END  OUPUT     ----\n")


    def log_error(self, text):
        """
        Log an error. This will be printed regardless of the status of the 
        verbose variable
        """
        print("\033[91m\033[1mError:\033[0m %s" % text, file=sys.stderr)
        




VERSION_NUMBER = "1.0.4-beta"

GYAT2PY_MAPPINGS = {
    "hawk": "try",
    "tuah": "except",
    "spit\s+on\s+that\s+thang": "finally",
    "its\s+giving": "return",
    "fanum\s+tax": "-",
    "rizz": "+",
    "yap": "print",
    "Aura": "True",
    "Cooked": "False",
    "bop": "def",
    "let\s+him\s+cook": "while",
    "glaze": "import",
    "lock\s+in": "from",
    "skibidi": "class",
    "chat\s+is\s+this\s+real": "if",
    "yo\s+chat": "elif",
    "only\s+in\s+ohio": "else",
    "mewing": "for",
    "just\s+put\s+the\s+fries\s+in\s+the\s+bag\s+bro": "break",
    "edge": "continue",
    "sus": "assert",
    "crashout": "raise",
    "pookie": "with",
    "ahh": "as",
    "GOAT": "global",
    "motion": "nonlocal",
    "delulu": "del",
    "pause no diddy": "yield from",
    "pause": "yield",
    "NPC": "None",
    "unc": "self",
    "huzz": "range",
    "sigma\s+twin": ">=",
    "beta\s+twin": "<=",
    "twin": "==",
    "sigma": ">",
    "beta": "<",
    "diddy": "in",
    "pluh": "pass"
}



from setuptools import setup
from pygyat import VERSION_NUMBER

with open("README.md", "r") as fh:
    long_description = fh.read()

# Install python package, scripts and manual pages
setup(name="pygyat",
      version=VERSION_NUMBER,
      author="Shamith Pasula",
      author_email="shamith09@berkeley.edu",
      license="MIT",
      description="Python with rizz",
      long_description=long_description,
      long_description_content_type="text/markdown",
      url="https://github.com/shamith09/pygyat",
      scripts=["scripts/gyat2py", "scripts/pygyat", "scripts/py2gyat"],
      data_files=[("man/man1", ["etc/pygyat.1", "etc/py2gyat.1", "etc/gyat2py.1"])],
      packages=["pygyat"],
      zip_safe=False)